@@ -2,6 +2,8 @@
 #ifndef _BCACHEFS_PROGRESS_H
 #define _BCACHEFS_PROGRESS_H
 
+#include "btree/bbpos_types.h"
+
 /*
  * Lame progress indicators
  *
@@ -13,30 +15,34 @@
  * that's not yet practical (mount).
  */
 
-struct progress_indicator_state {
+struct progress_indicator {
+	struct bbpos		pos;
 	unsigned long		next_print;
 	u64			nodes_seen;
 	u64			nodes_total;
 	struct btree		*last_node;
+	bool			silent;
 };
 
-void bch2_progress_init_inner(struct progress_indicator_state *s,
+void bch2_progress_init_inner(struct progress_indicator *s,
 			      struct bch_fs *c,
 			      u64 leaf_btree_id_mask,
 			      u64 inner_btree_id_mask);
 
-static inline void bch2_progress_init(struct progress_indicator_state *s,
+static inline void bch2_progress_init(struct progress_indicator *s,
 				      struct bch_fs *c, u64 btree_id_mask)
 {
 	bch2_progress_init_inner(s, c, btree_id_mask, 0);
 }
 
 int bch2_progress_update_iter(struct btree_trans *,
-			      struct progress_indicator_state *,
+			      struct progress_indicator *,
 			      struct btree_iter *,
 			      const char *);
 
 #define progress_update_iter(trans, p, iter)			\
 	bch2_progress_update_iter(trans, p, iter, __func__)
 
+void bch2_progress_to_text(struct printbuf *, struct progress_indicator *);
+
 #endif /* _BCACHEFS_PROGRESS_H */
