@@ -13,6 +13,8 @@
 #include "vfs/direct.h"
 #include "vfs/pagecache.h"
 
+#include "vendor/bio_iov_iter.h"
+
 #include "util/enumerated_ref.h"
 
 #include <linux/kthread.h>
@@ -86,7 +88,7 @@ static int bch2_direct_IO_read(struct kiocb *req, struct iov_iter *iter)
 
 	/* bios must be 512 byte aligned: */
 	if ((offset|iter->count) & (SECTOR_SIZE - 1))
-		return -EINVAL;
+		return bch_err_throw(c, unaligned_io);
 
 	ret = min_t(loff_t, iter->count,
 		    max_t(loff_t, 0, i_size_read(&inode->v) - offset));
@@ -148,11 +150,7 @@ start:
 		bio->bi_iter.bi_sector	= offset >> 9;
 		bio->bi_private		= dio;
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(6,17,0)
-		ret = bio_iov_iter_get_pages(bio, iter);
-#else
-		ret = bio_iov_iter_get_pages(bio, iter, 0);
-#endif
+		ret = bch2_bio_iov_iter_get_pages(bio, iter, 0);
 		if (ret < 0) {
 			/* XXX: fault inject this path */
 			bio->bi_status = BLK_STS_RESOURCE;
@@ -280,7 +278,7 @@ retry:
 
 		if (k.k->p.snapshot != snapshot ||
 		    nr_replicas > bch2_bkey_replicas(c, k) ||
-		    (!compressed && bch2_bkey_sectors_compressed(k)))
+		    (!compressed && bch2_bkey_sectors_compressed(c, k)))
 			return false;
 	}
 err:
@@ -465,11 +463,7 @@ static __always_inline long bch2_dio_write_loop(struct dio_write *dio)
 		EBUG_ON(current->faults_disabled_mapping);
 		current->faults_disabled_mapping = mapping;
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(6,17,0)
-		ret = bio_iov_iter_get_pages(bio, &dio->iter);
-#else
-		ret = bio_iov_iter_get_pages(bio, &dio->iter, 0);
-#endif
+		ret = bch2_bio_iov_iter_get_pages(bio, &dio->iter, 0);
 
 		dropped_locks = fdm_dropped_locks();
 
@@ -627,7 +621,7 @@ ssize_t bch2_direct_write(struct kiocb *req, struct iov_iter *iter)
 		goto err_put_write_ref;
 
 	if (unlikely((req->ki_pos|iter->count) & (block_bytes(c) - 1))) {
-		ret = -EINVAL;
+		ret = bch_err_throw(c, unaligned_io);
 		goto err_put_write_ref;
 	}
 
